#include "AppLogger_lua.h"
#include "AppLogger.h"
#include <stdlib.h>

extern "C" {
#include "lua.h"
#include "lualib.h"
#include "lauxlib.h"
}

static int AppLogger_setDebugLog(lua_State *L)
{

    AppLogger::setDebugLog(luaL_checkinteger(L,1), //mode
                           luaL_checkinteger(L,2)  //level
                          );
}

static int AppLogger_setSessionInterval(lua_State *L)
{

    AppLogger::setSessionInterval(luaL_checkinteger(L,1));
     return 0;
}

static int AppLogger_enableCrashReport(lua_State *L)
{
    AppLogger::enableCrashReport();
    return 0;
}

static int AppLogger_enableOnlineConfig(lua_State *L)
{
    logger_enableOnlineConfig();
    return 0;
}

static int AppLogger_getOnlineConfig(lua_State *L)
{
    size_t l;
    const char *key = luaL_checklstring(L, 1, &l);
    string value = AppLogger::getOnlineConfig(key);
    lua_pushstring(L, value.c_str());
    return 1;
}

static int AppLogger_onSubStart(lua_State *L)
{
    if (lua_type(L,1) == LUA_TSTRING)
    {
        size_t l;
        AppLogger::onSubStart(luaL_checklstring(L, 1, &l));//name
    }
    else if (lua_type(L,1) == LUA_TNUMBER)
    {
        char szName[128] = {0};
        sprintf(szName,"%td", luaL_checkinteger(L,1));
        AppLogger::onSubStart(szName);
    }

    return 0;
}

static int AppLogger_onSubEnd(lua_State *L)
{
    if (lua_type(L,1) == LUA_TSTRING)
    {
        size_t l;
        AppLogger::onSubEnd(luaL_checklstring(L, 1, &l));//name
    }
    else if (lua_type(L,1) == LUA_TNUMBER)
    {
        char szName[128] = {0};
        sprintf(szName,"%d", luaL_checkinteger(L,1));
        AppLogger::onSubEnd(szName);
    }

    return 0;
}

static int AppLogger_onPassFail(lua_State *L)
{
    AppLogger::onPassFail(luaL_checkinteger(L, 1));
    return 0;
}

static int AppLogger_onEvent(lua_State *L)
{

    size_t l;
    const char* name = luaL_checklstring(L, 1, &l);
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 1) {
        stage  = luaL_checklstring(L, 2, &l);
    }
    AppLogger::onEvent(name, stage);

    return 0;
}

static int AppLogger_onBuy(lua_State *L)
{
    size_t l;
    const char* stage = STAGE_LAST;
    float money = 0;
    
    if (lua_gettop(L) > 3) {
        if (lua_type(L,4) == LUA_TSTRING) {
            money = atof(luaL_checklstring(L, 4, &l));
        }
        else if (lua_type(L,4) == LUA_TNUMBER) {
            money = luaL_checkinteger(L, 4);
        }
    }
    if (lua_gettop(L) > 4) {
        stage  = luaL_checklstring(L, 5, &l);
    }
    
    AppLogger::onBuy(luaL_checklstring(L, 1, &l),  //service
                     luaL_checklstring(L, 2, &l),  //item
                     luaL_checkinteger(L, 3),      //count
                     money,                        //value
                     stage);                       //stage

    return 0;
}

static int AppLogger_onUse(lua_State *L)
{
    size_t l;
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 2) {
        stage  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::onUse(luaL_checklstring(L, 1, &l),  //item
                     luaL_checkinteger(L, 2),      //count
                     stage);                       //stage
    return 0;
}

static int AppLogger_onExchange(lua_State *L)
{
    size_t l;
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 2) {
        stage  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::onExchange(luaL_checklstring(L, 1, &l), //item
                          luaL_checkinteger(L, 2),     //count
                          stage);                      //stage
    return 0;
}

static int AppLogger_onCollect(lua_State *L)
{
    size_t l;
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 2) {
        stage  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::onCollect(luaL_checklstring(L, 1, &l), //item
                         luaL_checkinteger(L, 2),     //count
                         stage);                      //stage
    return 0;
}

static int AppLogger_onReward(lua_State *L)
{
    size_t l;
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 2) {
        stage  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::onReward(luaL_checklstring(L, 1, &l), //item
                        luaL_checkinteger(L, 2),     //count
                        stage);                      //stage
    return 0;
}

static int AppLogger_onShare(lua_State *L)
{	
    size_t l;
    const char* stage = STAGE_LAST;
    if (lua_gettop(L) > 2) {
        stage  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::onShare(luaL_checklstring(L, 1, &l), //service
                       luaL_checklstring(L, 2, &l), //item
                       stage );                     //stage
    return 0;
}

static int AppLogger_clearStatus(lua_State *L)
{
    AppLogger::clearStatus();
    return 0;
}

static int AppLogger_setStatus(lua_State *L)
{
    size_t l;
    const char *dist = NULL;
    if (lua_gettop(L) > 2) {
        dist  = luaL_checklstring(L, 3, &l);
    }
    AppLogger::setStatus(luaL_checklstring(L, 1, &l),  //key
                         luaL_checkinteger(L, 2),      //value
                         dist);                        //distribution

    return 0;
}

static int AppLogger_setUser(lua_State *L)
{
    size_t l;
    const char *level = luaL_checklstring(L, 1, &l);
    int age = -1;
    const char* gender = NULL;
    const char* userId = NULL;
    const char* userService = NULL;
    if (lua_gettop(L) > 1) {
        age = luaL_checkinteger(L, 2);
    }
    if (lua_gettop(L) > 2) {
        gender = luaL_checklstring(L, 3, &l);
    }
    if (lua_gettop(L) > 3) {
        userId = luaL_checklstring(L, 4, &l);
    }
    if (lua_gettop(L) > 4) {
        userService = luaL_checklstring(L, 5, &l);
    }
    AppLogger::setUser(level,       //level
                       age,         //age
                       gender,      //gender
                       userId,      //userId
                       userService);//userService
    return 0;
}


static luaL_Reg AppLogger_func[] = {
    {"setDebugLog",AppLogger_setDebugLog},
    {"setSessionInterval",AppLogger_setSessionInterval},
    {"enableCrashReport",AppLogger_enableCrashReport},
    {"enableOnlineConfig",AppLogger_enableOnlineConfig},
    {"getOnlineConfig",AppLogger_getOnlineConfig},
    {"onSubStart",AppLogger_onSubStart},
    {"onSubEnd",AppLogger_onSubEnd},
    {"onPassFail",AppLogger_onPassFail},
    {"onEvent",AppLogger_onEvent},
    {"onBuy",AppLogger_onBuy},
    {"onUse",AppLogger_onUse},
    {"onExchange",AppLogger_onExchange},
    {"onCollect",AppLogger_onCollect},
    {"onReward",AppLogger_onReward},
    {"onShare",AppLogger_onShare},
    {"clearStatus",AppLogger_clearStatus},
    {"setStatus",AppLogger_setStatus},
    {"setUser",AppLogger_setUser},
    {NULL,NULL}
};

#if LUA_VERSION_NUM > 501
//5.2.x
int AppLogger(lua_State *L)
{
    luaL_newlib(L,AppLogger_func);
    return 1;
}
void AppLogger_lua::init(lua_State* L)
{
    luaL_requiref(L, "AppLogger", AppLogger, 1);
}
#else
//5.1.x
void AppLogger_lua::init(lua_State* L)
{
    luaL_register(L,"AppLogger",AppLogger_func);
}
#endif